#include <PubSubClient.h>
#include <WiFi.h>
#include <FastLED.h>

const char* ssid     = "RPiHotspotN";
const char* password = "1234567890";
const char* mqttServer = "192.168.50.5";


int living_room_r = 255;
int living_room_g = 255;
int living_room_b = 255;

int kitchen_r = 255;
int kitchen_g = 255;
int kitchen_b = 255;

const int buzzer = 23;
bool buzzerOn = false;
int length = 15; // the number of notes
char notes[] = "ccggaagffeeddc "; // a space represents a rest
int beats[] = { 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 2, 4 };
int tempo = 300;

#define NUM_LEDS 2
#define DATA_PIN 22
CRGB leds[NUM_LEDS];

#define waterMotor 18
const int freq = 5000;
const int waterMotorChannel = 0;
const int resolution = 8;

int livingRoomLedButton = 3; //brown
int livingRoomLedState = LOW;
int livingRoomLedButtonState;
int livingRoomLedLastButtonState = LOW;

int kitchenLedButton = 21;// purple
int kitchenLedState = LOW;
int kitchenLedButtonState;
int kitchenLedLastButtonState = LOW;


int swimmingPoolButton = 5; //red
int swimmingPoolState = LOW;
int swimmingPoolButtonState;
int swimmingPoolLastButtonState = LOW;

unsigned long lastDebounceTime = 0;
unsigned long debounceDelay = 50;

WiFiClient espClient;
PubSubClient client(espClient);

void setup() {

  Serial.begin(115200);
  pinMode(buzzer, OUTPUT);

  setup_wifi_and_mqtt();

  Serial.println("pin mode setup...");
  pinMode(livingRoomLedButton, INPUT);
  pinMode(kitchenLedButton, INPUT);
  pinMode(swimmingPoolButton, INPUT);

  Serial.println("Fast LED...");
  FastLED.addLeds<WS2812, DATA_PIN, GRB>(leds, NUM_LEDS);

  Serial.println("Fast LED...Done");
  setupWaterMotor();

}



void setup_wifi_and_mqtt() {
  delay(10);
  unsigned long startedWaiting = millis();
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  int trial = 0;
  while (WiFi.status() != WL_CONNECTED && millis() - startedWaiting <= 10000) {
    delay(500);
    Serial.print(".");
    trial++;
    if (trial == 5) {
      break;
    }
  }

  if (WiFi.status() == WL_CONNECTED) {
    Serial.println("");
    Serial.println("WiFi connected");
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());

    Serial.println("Connecting to MQTT...");

    trial = 0;
    client.setServer(mqttServer, 1883);
    client.setCallback(callback);

    while (!client.connected()) {
      Serial.print("Attempting MQTT connection...");
      trial++;
      // Attempt to connect
      if (client.connect("right-house")) {
        Serial.println("connected");

        // Subscribe
        client.subscribe("house/bell_pressed");
        client.subscribe("house/kitchen");
        client.subscribe("house/living_room");
        client.subscribe("house/water_pump");

        client.subscribe("house/living_room_color_change");
        client.subscribe("house/kitchen_color_change");

      } else {
        Serial.print("failed, rc=");
        Serial.print(client.state());
        Serial.println(" try again in 5 seconds");
        // Wait 5 seconds before retrying
        delay(5000);
      }

      if (trial == 5) {
        break;
      }
    }
  }

}


void setupWaterMotor() {
  Serial.println("setting up water motor...");
  pinMode(waterMotor, OUTPUT);
  ledcSetup(waterMotorChannel, freq, resolution);
  ledcAttachPin(waterMotor, waterMotorChannel);

}

void callback(char* topic, byte* payload, unsigned int length) {

  Serial.print("Message arrived in topic: ");
  Serial.println(topic);

  Serial.print("Message:");
  String messageTemp;
  for (int i = 0; i < length; i++) {
    Serial.print((char)payload[i]);
    messageTemp += (char)payload[i];
  }


  if (String(topic) == "house/bell_pressed") {
    if (messageTemp == "1") {
      Serial.println("buzz");
      buzzerOn = true;
    }
  }


  if (String(topic) == "house/living_room") {
    Serial.println("Living room light on");
    if (messageTemp == "true") {
      leds[0] = CRGB(living_room_r, living_room_g, living_room_b);
    } else if (messageTemp == "false") {
      leds[0] = CRGB(0, 0, 0);
    }
  }


  if (String(topic) == "house/kitchen") {
    Serial.println("kitchen room light on");

    if (messageTemp == "true") {
      leds[1] = CRGB(kitchen_r, kitchen_g, kitchen_b);
    } else if (messageTemp == "false") {
      leds[1] = CRGB(0, 0, 0);
    }
  }

   if (String(topic) == "house/living_room_color_change") {
    long number = (long) strtol( &messageTemp[0], NULL, 16);
    living_room_r = number >> 16;
    living_room_g = number >> 8 & 0xFF;
    living_room_b = number & 0xFF;
    leds[0] = CRGB(living_room_r, living_room_g, living_room_b);
    client.publish("house/living_room_switch_on", "1");
  }

     if (String(topic) == "house/kitchen_color_change") {
    long number = (long) strtol( &messageTemp[0], NULL, 16);
    kitchen_r = number >> 16;
    kitchen_g = number >> 8 & 0xFF;
    kitchen_b = number & 0xFF;
    leds[1] = CRGB(kitchen_r, kitchen_g, kitchen_b);
    client.publish("house/kitchen_switch_on", "1");
  }

  FastLED.show();


  if (String(topic) == "house/water_pump") {
    Serial.println("water pump ");

    if (messageTemp == "true") {
      ledcWrite(waterMotorChannel , 1024);
    } else if (messageTemp == "false") {
      ledcWrite(waterMotorChannel , 0);
    }
  }

}

//
//void reconnect() {
//  // Loop until we're reconnected
//  while (!client.connected()) {
//    Serial.print("Attempting MQTT connection...");
//    // Attempt to connect
//    if (client.connect("right_house")) {
//      Serial.println("connected");
//
//      // Subscribe
//  client.subscribe("house/bell_pressed");
//  client.subscribe("house/kitchen");
//  client.subscribe("house/living_room");
//  client.subscribe("house/water_pump");
//
//
//    } else {
//      Serial.print("failed, rc=");
//      Serial.print(client.state());
//      Serial.println(" try again in 5 seconds");
//      // Wait 5 seconds before retrying
//      delay(5000);
//    }
//  }
//}
void loop() {
  //  if (!client.connected()) {
  //    reconnect();
  //  }


  if (buzzerOn) {
    for (int i = 0; i < length; i++) {
      if (notes[i] == ' ') {
        delay(beats[i] * tempo); // rest
      } else {
        playNote(notes[i], beats[i] * tempo);
      }

      // pause between notes
      delay(tempo / 2);
    }
    buzzerOn = false;
  }

  int livingRoomLedReading = digitalRead(livingRoomLedButton);
  if (livingRoomLedReading != livingRoomLedLastButtonState) {
    lastDebounceTime = millis();
  }

  if ((millis() - lastDebounceTime) + debounceDelay) {
    if (livingRoomLedReading != livingRoomLedButtonState) {
      livingRoomLedButtonState = livingRoomLedReading;

      // only toggle the LED if the new button state is HIGH
      if (livingRoomLedButtonState == HIGH) {
        livingRoomLedState = !livingRoomLedState;
        if (livingRoomLedState) {
          leds[0] = CRGB(255, 255, 255);
          client.publish("house/living_room_switch_on", "1");
        } else {
          leds[0] = CRGB(0, 0, 0);
          client.publish("house/living_room_switch_on", "0");
        }
      }
    }
  }

  livingRoomLedLastButtonState = livingRoomLedReading;
  FastLED.show();

  int kitchenLedReading = digitalRead(kitchenLedButton);
  if (kitchenLedReading != kitchenLedLastButtonState) {
    lastDebounceTime = millis();
  }

  if ((millis() - lastDebounceTime) + debounceDelay) {
    if (kitchenLedReading != kitchenLedButtonState) {
      kitchenLedButtonState = kitchenLedReading;

      // only toggle the LED if the new button state is HIGH
      if (kitchenLedButtonState == HIGH) {
        Serial.println(kitchenLedReading);
        kitchenLedState = !kitchenLedState;
        if (kitchenLedState) {
          leds[1] = CRGB(255, 255, 255);
          client.publish("house/kitchen_switch_on", "1");
        } else {
          leds[1] = CRGB(0, 0, 0);
          client.publish("house/kitchen_switch_on", "0");
        }
      }
    }
  }

  kitchenLedLastButtonState = kitchenLedReading;

  FastLED.show();


  int swimmingPoolReading = digitalRead(swimmingPoolButton);
  if (swimmingPoolReading != swimmingPoolLastButtonState) {
    lastDebounceTime = millis();
  }

  if ((millis() - lastDebounceTime) + debounceDelay) {
    if (swimmingPoolReading != swimmingPoolButtonState) {
      swimmingPoolButtonState = swimmingPoolReading;

      // only toggle the LED if the new button state is HIGH
      if (swimmingPoolButtonState == HIGH) {
        Serial.println(swimmingPoolReading);
        swimmingPoolState = !swimmingPoolState;
        if (swimmingPoolState) {
          ledcWrite(waterMotorChannel, 1024);
          client.publish("house/water_pump_switch_on", "1");
        } else {
          ledcWrite(waterMotorChannel, 0);
          client.publish("house/water_pump_switch_on", "0");
        }
      }
    }
  }

  swimmingPoolLastButtonState = swimmingPoolReading;

  client.loop();
}


void playTone(int tone, int duration) {
  for (long i = 0; i < duration * 1000L; i += tone * 2) {
    digitalWrite(buzzer, HIGH);
    delayMicroseconds(tone);
    digitalWrite(buzzer, LOW);
    delayMicroseconds(tone);
  }
}

void playNote(char note, int duration) {
  char names[] = { 'c', 'd', 'e', 'f', 'g', 'a', 'b', 'C' };
  int tones[] = { 1915, 1700, 1519, 1432, 1275, 1136, 1014, 956 };

  // play the tone corresponding to the note name
  for (int i = 0; i < 8; i++) {
    if (names[i] == note) {
      playTone(tones[i], duration);
    }
  }
}
